// LinkerNode.cpp
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include "DLLNode.h"

#include <Tools/FBuild/FBuildCore/FBuild.h>
#include <Tools/FBuild/FBuildCore/Graph/NodeGraph.h>

#include <Core/FileIO/IOStream.h>
#include <Core/Strings/AStackString.h>

// CONSTRUCTOR
//------------------------------------------------------------------------------
DLLNode::DLLNode( const AString & linkerOutputName,
				  const Array< Node * > & inputLibraries,
				  const AString & linker,
				  const AString & linkerArgs,
				  uint32_t flags )
: LinkerNode( linkerOutputName, inputLibraries, linker, linkerArgs, flags )
{
	m_Type = DLL_NODE;
}

// DESTRUCTOR
//------------------------------------------------------------------------------
DLLNode::~DLLNode()
{
}

// GetImportLibName
//------------------------------------------------------------------------------
void DLLNode::GetImportLibName( AString & importLibName ) const
{
	// get name minus extension (handle no extension gracefully)
	const char * lastDot = GetName().FindLast( '.' );
	lastDot = lastDot ? lastDot : GetName().GetEnd();
	importLibName.Assign( GetName().Get(), lastDot );

	// assume .lib extension for import
	importLibName += ".lib";
}

// Load
//------------------------------------------------------------------------------
/*static*/ Node * DLLNode::Load( IOStream & stream, bool remote )
{
	NODE_LOAD( AStackString<>,	name );
	NODE_LOAD( AStackString<>,	linker );
	NODE_LOAD( AStackString<>,	linkerArgs );
	NODE_LOAD_DEPS( 0,			inputLibs);
	NODE_LOAD( uint32_t,		flags );

	NodeGraph & ng = FBuild::Get().GetDependencyGraph();
	DLLNode * dn = ng.CreateDLLNode( name, inputLibs, linker, linkerArgs, flags );
	return dn;
}

//------------------------------------------------------------------------------
